local function size(t)
	local i = 0
	for k,v in pairs(t) do
		i = i + 1
	end
	return i
end

--------------------------------------------------------------
--Mounting and button click functions
--------------------------------------------------------------
function YayMounts:ShouldRepair()
	if(YayMountsState.forceRepairMount) then
		return true
	end
	
	local shouldRepair = false

	--Repair mount code
    if size(self.db.profile.misc.RepairMounts) > 0 and self.db.profile.settings["URM"] then
		--Check for low durability equipped gear
		for i = 0, 17 do
			local current, maximum = GetInventoryItemDurability(i)
			if current ~= nil and maximum ~= nil then
				if (current / maximum) <= self.db.profile.misc["MinDurability"] then
				shouldRepair = true
				end
			end
		end
      
		--Check inventory for low durability items
		if(not shouldRepair) then
			for i = 0, 4 do
				for j = 0, GetContainerNumSlots(i) do
					local current, maximum = GetContainerItemDurability(i, j);
					if current ~= nil and maximum ~= nil then
						if (current / maximum) <= self.db.profile.misc["MinDurability"] then
						shouldRepair = true
						end
					end
				end
			end
		end
    end
    
    return shouldRepair
end

function YayMounts:GetRepairMount()
	local i, e = 1, {}
  -- Try and use selected repair mounts first
	for k in pairs(self.db.profile.misc.RepairMounts) do
		if k then
			e[i] = k
			i = i + 1
		end
	end

	-- Use any repair mount if none selected
	if #e == 0 then 
		for k in pairs(self.db.profile.misc.RepairMounts) do
			e[i] = k
			i = i + 1
		end
	end
	
	return e
end

---- Helper method to get the passenger mounts
function YayMounts:BuildPassengerTable(mountType)
	local i, e = 1, {}
  
	for k,v in pairs(YayMountsType["Passenger"]) do
		local mountIndex = self.db.profile.mounts[k][3]
		if YayMounts:IsUsableMount(mountIndex) and v == mountType then
			e[i] = k
			i = i + 1
		end
	end
  
  -- If we wanted flying mounts, but have none, use any passenger mount
	if #e == 0 then
		for k,v in pairs(YayMountsType["Passenger"]) do
			local mountIndex = self.db.profile.mounts[k][3]
			if YayMounts:IsUsableMount(mountIndex) then
				e[i] = k
				i = i + 1
			end
		end
	end

	return e
end

function YayMounts:BuildStandardMountTable(mountType)
	local i, e = 1, {}
	local forceFlyingMounts = self.db.profile.settings["FFM"] and mountType == "ground" and IsUsableSpell(59976)
  
	for k,v in pairs(self.db.profile.mounts) do  
		local isCorrectType = forceFlyingMounts or v[2] == mountType
		if YayMounts:IsUsableMount(v[3]) and v[1] and isCorrectType then
			e[i] = k
			i = i + 1
		end
	end
  
	return e
end

function YayMounts:Mounting(flyable, zone)
	if not self.db.profile.settings["HM"] then
		return nil
	end
		
	local e = {}
	local useRepairMount = 1, false
	local mountType
	
	if flyable then
		mountType = "fly"
	else
		mountType = "ground"
	end
	
	--Repair mount
	if YayMounts:ShouldRepair() then
		e = YayMounts:GetRepairMount()
	elseif YayMountsState.usePassengerMount then
		e = YayMounts:BuildPassengerTable(mountType)
	else
		e = YayMounts:BuildStandardMountTable(mountType)
	end
	
	-- Filter out mounts for certain zones (AQ, etc)
	e = YayMounts:Filter(e, zone)
	
	-- Load emergency mounts if we have no usable mounts yet
	if #e == 0 and self.db.profile.settings["ER"] then
		if(mountType == "fly" and self.db.profile.misc.NumMounts.Flying == 0) then
			mountType = "ground"
		end
		
		local i = 1
		for spellID,v in pairs(self.db.profile.mounts) do
			if YayMounts:IsUsableMount(v[3]) and v[2] == mountType then
				e[i] = spellID
				i = i + 1
			end
		end
	end
	
	-- Final check to remove any currently unusable mounts
	for k,spellId in pairs(e) do
		if not YayMounts:IsUsableMount(self.db.profile.mounts[spellId][3]) then
			tremove(e, k)
		end
	end
	
	if #e == 0 then
		self:Print(YayMountsLocale.String.NoMounts)
		return nil
	end
	
	return YayMounts:SpellToName(e[math.random(#e)])
end

function YayMounts:PreClick()
	if YayMountsState.playerCombat then
		return
	end

	local _,_,WGActive,_,_,_ = GetWorldPVPAreaInfo(1)
	local isFlyable
	local broom = false
	local zone = GetRealZoneText()
	local loanedMount = YayMounts:CheckLoanerMount()

	-- Check if we can fly
	isFlyable = IsFlyableArea()
	
	--Check if we're in Draenor for flying
	if((GetCurrentMapContinent() == 7 or zone == "Tanaan Jungle") and not IsSpellKnown(191645)) then
		isFlyable = false
		if(zone == "Nagrand" and self.db.profile.settings["TELAARI"] == true) then
			if(YayMounts:CheckNagrandSpecialMount()) then --Telaari Talbuk/Horde wolf
				return
			end
		end
	end

	--Check if in Wintergrasp
	if zone == YayMountsLocale.Zone.Wintergrasp then 
		isFlyable = not WGActive
	end
     
	-- check if we can use flying broom
	if GetItemCount(37011, false) > 0 and not YayMountsState.playerCombat and self.db.profile.settings["FBP"] == true then
		broom = true
	end
	
  ------------Dismounting logic-------------
	if IsMounted() and IsFlying() then
		if YayMountsState.class == "DRUID" and self.db.profile.settings["MTFF"] and GetUnitSpeed("player") ~= 0 then 
			YayMounts:Druids(isFlyable, zone) -- Druid: Flying mount to flight form   
		elseif self.db.profile.settings["NDWF"] then
			YayMounts:SetMacroButton()
		else
			Dismount()
			YayMounts:SetMacroButton()
		end
  
	elseif IsMounted() then
		if IsSwimming() and YayMountsState.class == "DRUID" and IsUsableSpell(783) and GetUnitSpeed("player") ~= 0  then
			YayMounts:SetMacroButton(YayMounts:SpellToName(783)) -- Aquatic Form  
		else
			Dismount()
			YayMounts:SetMacroButton()
		end
    
	elseif CanExitVehicle() then
		YayMounts:SetMacroButton()
		VehicleExit()
    
	------------Not currently mounted-------------
	elseif YayMounts:ClassCheck() or YayMounts:PriestOrMageSpecial() then
		return
    
	elseif isFlyable and IsOutdoors() then
		if broom then --flying broom
			YayMounts:SetMacroButton(YayMounts:ItemToName(37011))
		elseif IsSwimming() then
			YayMounts:Swimming(isFlyable, zone, broom)
		elseif YayMountsState.class == "DRUID" then
			YayMounts:Druids(isFlyable, zone)
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(isFlyable, zone))
		end
    
	elseif not isFlyable and IsOutdoors() and zone ~= YayMountsLocale.Zone.VortexPinnacle then
		if zone == YayMountsLocale.Zone.Oculus and YayMounts:Filter(nil, zone) then
			return
		elseif broom then --flying broom
			YayMounts:SetMacroButton(YayMounts:ItemToName(37011))     
		elseif IsSwimming() then
			YayMounts:Swimming(isFlyable, zone, broom)
		elseif loanedMount ~= nil and not YayMountsState.playerCombat then
			YayMounts:SetMacroButton(YayMounts:ItemToName(loanedMount))
		elseif YayMountsState.class == "DRUID" then
			YayMounts:Druids(isFlyable, zone)
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(isFlyable, zone))
		end
    
	elseif not IsOutdoors() or zone == YayMountsLocale.Zone.VortexPinnacle then
		if IsSwimming() then
			YayMounts:Swimming(isFlyable, zone, broom)
		elseif YayMountsState.class == "DRUID" and IsUsableSpell(768) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(768)) -- Cat form
		end
    
	else
		--No logic left, just try to use a mount
		YayMounts:SetMacroButton(YayMounts:Mounting(isFlyable, zone)) 
	end
end

function YayMounts:ForcePassengerMount()
	YayMountsState.usePassengerMount = true
	YayMounts:PreClick()
	YayMountsState.usePassengerMount = false
end

function YayMounts:ForceRepairMount()
	YayMountsState.forceRepairMount = true
	YayMounts:PreClick()
	YayMountsState.forceRepairMount = false
end

-- Tries to use the Nagrand outpost special ability
function YayMounts:CheckNagrandSpecialMount()
	local garrisonAbilityName = GetSpellInfo(161691)
	local _,_,_,_,_,_,spellID = GetSpellInfo(garrisonAbilityName)
	if(spellID == 165803 or spellID == 164222) then -- Have the proper outpost
		if(YayMountsState.playerCombat) then
			YayMounts:SetMacroButtonCombat(garrisonAbilityName)
		else
			YayMounts:SetMacroButton(garrisonAbilityName)
		end
		return true
	else
		return false
	end
end

-- Checks if slowfall or levitate should be used
function YayMounts:PriestOrMageSpecial()
	if not YayMountsState.playerCombat then
		if (IsFalling() or GetUnitSpeed("player") ~= 0) and YayMountsState.class == "PRIEST" and IsUsableSpell(1706) and self.db.profile.settings["PRIEST"] then --IsUsableSpell checks for reagents/glyphs
			YayMounts:SetMacroButton("[@player] "..YayMounts:SpellToName(1706).."\n/cancelaura "..YayMounts:SpellToName(1706)) --Levitate
			return true
		  
		elseif (IsFalling() or GetUnitSpeed("player") ~= 0) and YayMountsState.class == "MAGE" and IsUsableSpell(130) and self.db.profile.settings["MAGE"] then
			YayMounts:SetMacroButton("[@player] "..YayMounts:SpellToName(130).."\n/cancelaura "..YayMounts:SpellToName(130)) --Slow fall
			return true
		  
		else
			return false
		end
	end
end

function YayMounts:ClassCheck()
	if not IsUsableSpell(YMSpellIDs[YayMountsState.class]) then
		return false
	end

	if GetUnitSpeed("player") ~= 0 and self.db.profile.settings[YayMountsState.class] then
		YayMounts:SetMacroButton(YayMounts:SpellToName(YMSpellIDs[YayMountsState.class]))
		return true
	end
	
	return false
end

function YayMounts:Druids(flyable, zone)
	local travelForm, flightForm = 783, 783 -- 3 in 1 travel form

	if GetUnitSpeed("player") ~= 0 then
		if flyable then
			YayMounts:SetMacroButton(YayMounts:SpellToName(flightForm))
		else
			YayMounts:SetMacroButton(YayMounts:SpellToName(travelForm))
		end
		
	elseif(flyable and not YayMountsState.playerCombat) then
		if self.db.profile.settings["FFP"] or IsFalling() or IsFlying() or GetShapeshiftForm() == 3 then
			YayMounts:SetMacroButton(YayMounts:SpellToName(flightForm))
		else
			YayMounts:SetMacroButton(YayMounts:Mounting(flyable, zone), 1)
		end
    
	else
		YayMounts:SetMacroButton(YayMounts:Mounting(false, zone), 1)  
	end
end

function YayMounts:Swimming(flyable, zone, broom)

	local aquaticForm = 783

	local timer, _,_,scale,_,_ = GetMirrorTimerInfo(2)
	local name, _, _, _, _, _, _, _, _, _, _ = UnitBuff("player", YayMountsLocale.Misc.WaterBreathing)
	if timer == "UNKNOWN" or scale >= 0 then
		scale = 1
	end
  
	if flyable and not YayMountsState.playerCombat then
	if YayMountsState.class == "DRUID" and IsUsableSpell(aquaticForm) and GetUnitSpeed("player") ~= 0 then
		YayMounts:SetMacroButton(YayMounts:SpellToName(aquaticForm)) --Aquatic Form
	elseif YayMountsState.class == "DRUID" then
		YayMounts:Druids(flyable, zone)
	else
		YayMounts:SetMacroButton(YayMounts:Mounting(true, zone))
	end
    
	elseif IsOutdoors() and not YayMountsState.playerCombat then
		if YayMountsState.class == "DRUID" and IsUsableSpell(aquaticForm) and GetUnitSpeed("player") ~= 0 then
			YayMounts:SetMacroButton(YayMounts:SpellToName(aquaticForm)) --Aquatic Form
		else 
			if zone == YayMountsLocale.Zone.Vashjir1 or zone == YayMountsLocale.Zone.Vashjir2 or zone == YayMountsLocale.Zone.Vashjir3 or zone == YayMountsLocale.Zone.Vashjir4 or zone == YayMountsLocale.Zone.Vashjir5 and self.db.profile.mounts[75207][1] == true then --Abyssal Seahorse
				YayMounts:SetMacroButton(YayMounts:SpellToName(75207)) --Abyssal Seahorse
			elseif IsUsableSpell(98718) and self.db.profile.mounts[98718][1] == true then
				YayMounts:SetMacroButton(YayMounts:SpellToName(98718)) --Subdued Seahorse
			elseif IsUsableSpell(64731) and self.db.profile.mounts[64731][1] then
				YayMounts:SetMacroButton(YayMounts:SpellToName(64731)) --Sea Turtle
			else
				YayMounts:SetMacroButton(YayMounts:Mounting(false, zone))
			end
		end
    
	elseif not YayMountsState.playerCombat then
		if YayMountsState.class == "DRUID" and IsUsableSpell(aquaticForm) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(aquaticForm)) -- Aquatic Form
		elseif YayMountsState.class == "SHAMAN" and IsUsableSpell(2645) then
			YayMounts:SetMacroButton(YayMounts:SpellToName(2645)) --Ghost wolf
		end
	end
end

--check if we have cold weather flying, a loaned mount, and/or flight master's license
function YayMounts:CheckLoanerMount()
  local zone = GetRealZoneText()
  if zone == YayMountsLocale.Zone.StormPeaks or zone == YayMountsLocale.Zone.Icecrown then
    if GetItemCount(44221, false) > 0 then
      return 44221
    elseif GetItemCount(44229, false) > 0 then
      return 44229
    else
      return nil
    end
  end
  return nil
end

function YayMounts:SetMacroButton(mount, button)
	if not YayMountsState.playerCombat then
		if mount and button == 2 then
			YayMountsButton2:SetAttribute("macrotext", "/use "..YayMounts:Mounting(false, GetRealZoneText()))
		elseif mount and YayMountsState.usePassengerMount then
			YayMountsButton3:SetAttribute("macrotext", "/use "..mount)
		elseif mount and YayMountsState.forceRepairMount then
			YayMountsButton4:SetAttribute("macrotext", "/use "..mount)
		elseif mount then
			YayMountsButton:SetAttribute("macrotext", "/use "..mount)   
		else
			YayMountsButton:SetAttribute("macrotext", nil)
		end
	end
end

function YayMounts:SetMacroButtonCombat(mount, button)
	if mount and button == 2 then
		YayMountsButton2:SetAttribute("macrotext", "/use "..YayMounts:Mounting(false, GetRealZoneText()))
	elseif mount and button == 3 then
		YayMountsButton3:SetAttribute("macrotext", "/use "..mount)
	elseif mount then
		YayMountsButton:SetAttribute("macrotext", "/use "..mount)
	else
		YayMountsButton:SetAttribute("macrotext", nil)
    end
end

function YayMounts:Filter(e, zone)
  if zone ~= YayMountsLocale.Zone.AQ40 and e ~= nil then
    for k,v in pairs(e) do
      if v == YayMountsType["Ahn'Qiraj"][v] then
        tremove(e, k)
      end
    end
    return e
    
  elseif zone == YayMountsLocale.Zone.AQ40 and e ~= nil then
    local aq40 = false
    local aq40Table = {}
    local i = 1
    
    for k,v in pairs(e) do
      if v == YayMountsType["Ahn'Qiraj"][v] then
        aq40 = true
        
        break
      end
    end
    
    if aq40 then
      for k,v in pairs(e) do
        if v == YayMountsType["Ahn'Qiraj"][v] then
          aq40Table[i] = v
          i = i + 1
        end
      end
      return aq40Table
    
    else
      return e
    end
    
  
  elseif zone == YayMountsLocale.Zone.Oculus and e == nil then
    if GetItemCount(37859) == 1 then
      YayMounts:SetMacroButton(YayMounts:ItemToName(37859))
      return true
    elseif GetItemCount(37860) == 1 then
      YayMounts:SetMacroButton(YayMounts:ItemToName(37860))
      return true
    elseif GetItemCount(37815) == 1 then
      YayMounts:SetMacroButton(YayMounts:ItemToName(37815))
      return true
    end
    return false
    
  else
    return e
  end
end
